@@ -29,6 +29,10 @@
  * SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifdef _KERNEL
 #if defined(__FreeBSD__) && __FreeBSD__ >= 3
 #include "opt_inet.h"
@@ -50,7 +54,11 @@
 #include <sys/socket.h>
 
 #include <netinet/in.h>
-#include <netinet/ipsec.h>
+#ifdef HAVE_NETINET6_IPSEC
+#  include <netinet6/ipsec.h>
+#else
+#  include <netinet/ipsec.h>
+#endif
 
 #ifndef _KERNEL
 #include <ctype.h>
@@ -58,6 +66,7 @@
 #include <stdlib.h>
 #endif /* !_KERNEL */
 
+#include "config.h"
 #include "libpfkey.h"
 
 static void kdebug_sadb_prop __P((struct sadb_ext *));
@@ -265,7 +274,7 @@ kdebug_sadb_identity(ext)
 			p = (char *)(id + 1);
 			ep = p + len;
 			for (/*nothing*/; *p && p < ep; p++) {
-				if (isprint(*p))
+				if (isprint((int)*p))
 					printf("%c", *p & 0xff);
 				else
 					printf("\\%03o", *p & 0xff);
@@ -425,9 +434,17 @@ kdebug_sadb_x_policy(ext)
 	if (ext == NULL)
 		panic("kdebug_sadb_x_policy: NULL pointer was passed.\n");
 
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+	printf("sadb_x_policy{ type=%u dir=%u id=%x priority=%u }\n",
+#else
 	printf("sadb_x_policy{ type=%u dir=%u id=%x }\n",
+#endif
 		xpl->sadb_x_policy_type, xpl->sadb_x_policy_dir,
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+		xpl->sadb_x_policy_id, xpl->sadb_x_policy_priority);
+#else
 		xpl->sadb_x_policy_id);
+#endif
 
 	if (xpl->sadb_x_policy_type == IPSEC_POLICY_IPSEC) {
 		int tlen;
