@@ -62,7 +62,7 @@ static int pfkey_send_x1 __P((int, u_int, u_int, u_int, struct sockaddr *,
 	struct sockaddr *, u_int32_t, u_int32_t, u_int, caddr_t,
 	u_int, u_int, u_int, u_int, u_int, u_int32_t, u_int32_t,
 	u_int32_t, u_int32_t, u_int32_t,
-	u_int8_t, u_int16_t, u_int16_t, struct sockaddr *));
+	u_int8_t, u_int16_t, u_int16_t, struct sockaddr *, u_int16_t));
 static int pfkey_send_x2 __P((int, u_int, u_int, u_int,
 	struct sockaddr *, struct sockaddr *, u_int32_t));
 static int pfkey_send_x3 __P((int, u_int, u_int));
@@ -86,16 +86,26 @@ static caddr_t pfkey_setsadbxsa2 __P((caddr_t, caddr_t, u_int32_t, u_int32_t));
 static caddr_t pfkey_set_natt_type __P((caddr_t, caddr_t, u_int, u_int8_t));
 static caddr_t pfkey_set_natt_port __P((caddr_t, caddr_t, u_int, u_int16_t));
 #endif
+#ifdef SADB_X_EXT_NAT_T_FRAG
+static caddr_t pfkey_set_natt_frag __P((caddr_t, caddr_t, u_int, u_int16_t));
+#endif
 
 /*
  * make and search supported algorithm structure.
  */
-static struct sadb_supported *ipsec_supported[] = { NULL, NULL, NULL, };
+static struct sadb_supported *ipsec_supported[] = { NULL, NULL, NULL, 
+#ifdef SADB_X_SATYPE_TCPSIGNATURE
+    NULL,
+#endif
+};
 
 static int supported_map[] = {
 	SADB_SATYPE_AH,
 	SADB_SATYPE_ESP,
 	SADB_X_SATYPE_IPCOMP,
+#ifdef SADB_X_SATYPE_TCPSIGNATURE
+	SADB_X_SATYPE_TCPSIGNATURE,
+#endif
 };
 
 static int
@@ -500,7 +510,7 @@ pfkey_send_update(so, satype, mode, src, dst, spi, reqid, wsize,
 			reqid, wsize,
 			keymat, e_type, e_keylen, a_type, a_keylen, flags,
 			l_alloc, l_bytes, l_addtime, l_usetime, seq,
-			0, 0, 0, NULL)) < 0)
+			0, 0, 0, NULL, 0)) < 0)
 		return -1;
 
 	return len;
@@ -511,7 +521,8 @@ int
 pfkey_send_update_nat(so, satype, mode, src, dst, spi, reqid, wsize,
 		      keymat, e_type, e_keylen, a_type, a_keylen, flags,
 		      l_alloc, l_bytes, l_addtime, l_usetime, seq,
-		      l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa)
+		      l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa,
+		      l_natt_frag)
 	int so;
 	u_int satype, mode, wsize;
 	struct sockaddr *src, *dst;
@@ -524,13 +535,15 @@ pfkey_send_update_nat(so, satype, mode, src, dst, spi, reqid, wsize,
 	u_int8_t l_natt_type;
 	u_int16_t l_natt_sport, l_natt_dport;
 	struct sockaddr *l_natt_oa;
+	u_int16_t l_natt_frag;
 {
 	int len;
 	if ((len = pfkey_send_x1(so, SADB_UPDATE, satype, mode, src, dst, spi,
 			reqid, wsize,
 			keymat, e_type, e_keylen, a_type, a_keylen, flags,
 			l_alloc, l_bytes, l_addtime, l_usetime, seq,
-			l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa)) < 0)
+			l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa,
+			l_natt_frag)) < 0)
 		return -1;
 
 	return len;
@@ -563,7 +576,7 @@ pfkey_send_add(so, satype, mode, src, dst, spi, reqid, wsize,
 			reqid, wsize,
 			keymat, e_type, e_keylen, a_type, a_keylen, flags,
 			l_alloc, l_bytes, l_addtime, l_usetime, seq,
-			0, 0, 0, NULL)) < 0)
+			0, 0, 0, NULL, 0)) < 0)
 		return -1;
 
 	return len;
@@ -574,7 +587,8 @@ int
 pfkey_send_add_nat(so, satype, mode, src, dst, spi, reqid, wsize,
 		   keymat, e_type, e_keylen, a_type, a_keylen, flags,
 		   l_alloc, l_bytes, l_addtime, l_usetime, seq,
-		   l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa)
+		   l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa,
+		   l_natt_frag)
 	int so;
 	u_int satype, mode, wsize;
 	struct sockaddr *src, *dst;
@@ -587,13 +601,15 @@ pfkey_send_add_nat(so, satype, mode, src, dst, spi, reqid, wsize,
 	u_int8_t l_natt_type;
 	u_int16_t l_natt_sport, l_natt_dport;
 	struct sockaddr *l_natt_oa;
+	u_int16_t l_natt_frag;
 {
 	int len;
 	if ((len = pfkey_send_x1(so, SADB_ADD, satype, mode, src, dst, spi,
 			reqid, wsize,
 			keymat, e_type, e_keylen, a_type, a_keylen, flags,
 			l_alloc, l_bytes, l_addtime, l_usetime, seq,
-			l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa)) < 0)
+			l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa,
+			l_natt_frag)) < 0)
 		return -1;
 
 	return len;
@@ -1183,7 +1199,8 @@ static int
 pfkey_send_x1(so, type, satype, mode, src, dst, spi, reqid, wsize,
 		keymat, e_type, e_keylen, a_type, a_keylen, flags,
 		l_alloc, l_bytes, l_addtime, l_usetime, seq,
-	        l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa)
+	        l_natt_type, l_natt_sport, l_natt_dport, l_natt_oa, 
+		l_natt_frag)
 	int so;
 	u_int type, satype, mode;
 	struct sockaddr *src, *dst, *l_natt_oa;
@@ -1194,6 +1211,7 @@ pfkey_send_x1(so, type, satype, mode, src, dst, spi, reqid, wsize,
 	u_int32_t l_alloc, l_bytes, l_addtime, l_usetime, seq;
 	u_int16_t l_natt_sport, l_natt_dport;
 	u_int8_t l_natt_type;
+	u_int16_t l_natt_frag;
 {
 	struct sadb_msg *newmsg;
 	int len;
@@ -1249,6 +1267,18 @@ pfkey_send_x1(so, type, satype, mode, src, dst, spi, reqid, wsize,
 			return -1;
 		}
 		break;
+#ifdef SADB_X_AALG_TCP_MD5
+	case SADB_X_SATYPE_TCPSIGNATURE:
+		if (e_type != SADB_EALG_NONE) {
+			__ipsec_errcode = EIPSEC_INVAL_ALGS;
+			return -1;
+		}
+		if (a_type != SADB_X_AALG_TCP_MD5) {
+			__ipsec_errcode = EIPSEC_INVAL_ALGS;
+			return -1;
+		}
+		break;
+#endif
 	default:
 		__ipsec_errcode = EIPSEC_INVAL_SATYPE;
 		return -1;
@@ -1273,9 +1303,14 @@ pfkey_send_x1(so, type, satype, mode, src, dst, spi, reqid, wsize,
 #ifdef SADB_X_EXT_NAT_T_TYPE
 	/* add nat-t packets */
 	if (l_natt_type) {
-		if (satype != SADB_SATYPE_ESP) {
+		switch(satype) {
+		case SADB_SATYPE_ESP:
+		case SADB_X_SATYPE_IPCOMP:
+			break;
+		default:
 			__ipsec_errcode = EIPSEC_NO_ALGS;
 			return -1;
+			break;
 		}
 
 		len += sizeof(struct sadb_x_nat_t_type);
@@ -1284,6 +1319,10 @@ pfkey_send_x1(so, type, satype, mode, src, dst, spi, reqid, wsize,
 		if (l_natt_oa)
 			len += sizeof(struct sadb_address) +
 			  PFKEY_ALIGN8(sysdep_sa_len(l_natt_oa));
+#ifdef SADB_X_EXT_NAT_T_FRAG
+		if (l_natt_frag)
+			len += sizeof(struct sadb_x_nat_t_frag);
+#endif
 	}
 #endif
 
@@ -1386,6 +1425,17 @@ pfkey_send_x1(so, type, satype, mode, src, dst, spi, reqid, wsize,
 				return -1;
 			}
 		}
+
+		if (l_natt_frag) {
+#ifdef SADB_X_EXT_NAT_T_FRAG
+			p = pfkey_set_natt_frag(p, ep, SADB_X_EXT_NAT_T_FRAG,
+					l_natt_frag);
+			if (!p) {
+				free(newmsg);
+				return -1;
+			}
+#endif
+		}
 	}
 #endif
 
@@ -1517,6 +1567,9 @@ pfkey_send_x3(so, type, satype)
 		case SADB_SATYPE_AH:
 		case SADB_SATYPE_ESP:
 		case SADB_X_SATYPE_IPCOMP:
+#ifdef SADB_X_SATYPE_TCPSIGNATURE
+		case SADB_X_SATYPE_TCPSIGNATURE:
+#endif
 			break;
 		default:
 			__ipsec_errcode = EIPSEC_INVAL_SATYPE;
@@ -1988,6 +2041,9 @@ pfkey_check(mhp)
 	case SADB_SATYPE_ESP:
 	case SADB_SATYPE_AH:
 	case SADB_X_SATYPE_IPCOMP:
+#ifdef SADB_X_SATYPE_TCPSIGNATURE
+	case SADB_X_SATYPE_TCPSIGNATURE:
+#endif
 		switch (msg->sadb_msg_type) {
 		case SADB_X_SPDADD:
 		case SADB_X_SPDDELETE:
@@ -2317,3 +2373,29 @@ pfkey_set_natt_port(buf, lim, type, l_natt_port)
 	return(buf + len);
 }
 #endif
+
+#ifdef SADB_X_EXT_NAT_T_FRAG
+static caddr_t
+pfkey_set_natt_frag(buf, lim, type, l_natt_frag)
+	caddr_t buf;
+	caddr_t lim;
+	u_int type;
+	u_int16_t l_natt_frag;
+{
+	struct sadb_x_nat_t_frag *p;
+	u_int len;
+
+	p = (struct sadb_x_nat_t_frag *)buf;
+	len = sizeof(struct sadb_x_nat_t_frag);
+
+	if (buf + len > lim)
+		return NULL;
+
+	memset(p, 0, len);
+	p->sadb_x_nat_t_frag_len = PFKEY_UNIT64(len);
+	p->sadb_x_nat_t_frag_exttype = type;
+	p->sadb_x_nat_t_frag_fraglen = l_natt_frag;
+
+	return(buf + len);
+}
+#endif
