@@ -40,6 +40,7 @@
 #include <sys/queue.h>
 #include <sys/socket.h>
 #include <netdb.h>
+#include <unistd.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -92,7 +93,7 @@ gssapi_error(OM_uint32 status_code, const char *where,
 			    "UNABLE TO GET GSSAPI ERROR CODE\n");
 		else {
 			plog(LLV_ERROR, where, NULL,
-			    "%s\n", status_string.value);
+			    "%s\n", (char *)status_string.value);
 			gss_release_buffer(&min_stat, &status_string);
 		}
 	} while (message_context != 0);
@@ -128,6 +129,25 @@ gssapi_gss2vmbuf(gss_buffer_t gsstoken, vchar_t **vmbuf)
 	return 0;
 }
 
+vchar_t *
+gssapi_get_default_gss_id(void)
+{
+	char name[NI_MAXHOST];
+	vchar_t *gssid;
+
+	if (gethostname(name, sizeof(name)) != 0) {
+		plog(LLV_ERROR, LOCATION, NULL, "gethostname failed: %s\n",
+		    strerror(errno));
+		return (NULL);
+	}
+	name[sizeof(name) - 1] = '\0';
+
+	gssid = racoon_malloc(sizeof(*gssid));
+	gssid->l = asprintf(&gssid->v, "%s/%s", GSSAPI_DEF_NAME, name);
+
+	return (gssid);
+}
+
 static int
 gssapi_get_default_name(struct ph1handle *iph1, int remote, gss_name_t *service)
 {
@@ -216,8 +236,15 @@ gssapi_init(struct ph1handle *iph1)
 		return -1;
 	}
 
-	plog(LLV_DEBUG, LOCATION, NULL, "will try to acquire '%*s' creds\n",
+#if 0
+	/*
+	 * XXXJRT Did this debug message ever work?  This is a GSS name
+	 * blob at this point.
+	 */
+	plog(LLV_DEBUG, LOCATION, NULL, "will try to acquire '%.*s' creds\n",
 	    cred->length, cred->value);
+#endif
+
 	maj_stat = gss_release_buffer(&min_stat, cred);
 	if (GSS_ERROR(maj_stat))
 		gssapi_error(min_stat, LOCATION, "release cred buffer\n");
@@ -259,15 +286,16 @@ gssapi_get_itoken(struct ph1handle *iph1, int *lenp)
 	dummy = &empty;
 
 	if (iph1->approval != NULL && iph1->approval->gssid != NULL) {
-		plog(LLV_DEBUG, LOCATION, NULL, "using provided service '%s'\n",
-		    iph1->approval->gssid->v);
+		plog(LLV_DEBUG, LOCATION, NULL,
+		    "using provided service '%.*s'\n",
+		    iph1->approval->gssid->l, iph1->approval->gssid->v);
 		name_token.length = iph1->approval->gssid->l;
 		name_token.value = iph1->approval->gssid->v;
 		maj_stat = gss_import_name(&min_stat, &name_token,
 		    GSS_C_NO_OID, &partner);
 		if (GSS_ERROR(maj_stat)) {
-			gssapi_error(min_stat, LOCATION, "import of %s\n",
-			    name_token.value);
+			gssapi_error(min_stat, LOCATION, "import of %.*s\n",
+			    name_token.length, name_token.value);
 			return -1;
 		}
 	} else
@@ -351,7 +379,7 @@ gssapi_get_rtoken(struct ph1handle *iph1, int *lenp)
 
 	plog(LLV_DEBUG, LOCATION, NULL,
 		"gss_accept_sec_context: other side is %s\n",
-		name_token.value);
+		(char *)name_token.value);
 	maj_stat = gss_release_buffer(&min_stat, &name_token);
 	if (GSS_ERROR(maj_stat))
 		gssapi_error(min_stat, LOCATION, "release name buffer\n");
@@ -649,7 +677,7 @@ gssapi_free_state(struct ph1handle *iph1)
 }
 
 vchar_t *
-gssapi_get_default_id(struct ph1handle *iph1)
+gssapi_get_id(struct ph1handle *iph1)
 {
 	gss_buffer_desc id_buffer;
 	gss_buffer_t id = &id_buffer;
@@ -657,6 +685,9 @@ gssapi_get_default_id(struct ph1handle *iph1)
 	OM_uint32 min_stat, maj_stat;
 	vchar_t *vmbuf;
 
+	if (iph1->rmconf->proposal->gssid != NULL)
+		return (vdup(iph1->rmconf->proposal->gssid));
+
 	if (gssapi_get_default_name(iph1, 0, &defname) < 0)
 		return NULL;
 
@@ -687,8 +718,14 @@ gssapi_get_default_id(struct ph1handle *iph1)
 	if (GSS_ERROR(maj_stat))
 		gssapi_error(min_stat, LOCATION, "release canonical name\n");
 
-	plog(LLV_DEBUG, LOCATION, NULL, "will try to acquire '%*s' creds\n",
+#if 0
+	/*
+	 * XXXJRT Did this debug message ever work?  This is a GSS name
+	 * blob at this point.
+	 */
+	plog(LLV_DEBUG, LOCATION, NULL, "will try to acquire '%.*s' creds\n",
 	    id->length, id->value);
+#endif
 
 	if (gssapi_gss2vmbuf(id, &vmbuf) < 0) {
 		plog(LLV_ERROR, LOCATION, NULL, "gss2vmbuf failed\n");
