@@ -1,4 +1,4 @@
-/* $Id: handler.c,v 1.13 2004/11/21 19:36:26 manubsd Exp $ */
+/* $Id: handler.c,v 1.13.4.3 2005/05/20 00:37:41 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -138,6 +138,23 @@ getph1byaddr(local, remote)
 {
 	struct ph1handle *p;
 
+	LIST_FOREACH(p, &ph1tree, chain) {
+		if (p->status == PHASE1ST_EXPIRED)
+			continue;
+		if (CMPSADDR(local, p->local) == 0
+		 && CMPSADDR(remote, p->remote) == 0)
+			return p;
+	}
+
+	return NULL;
+}
+
+struct ph1handle *
+getph1byaddrwop(local, remote)
+	struct sockaddr *local, *remote;
+{
+	struct ph1handle *p;
+
 	LIST_FOREACH(p, &ph1tree, chain) {
 		if (p->status == PHASE1ST_EXPIRED)
 			continue;
@@ -155,7 +172,7 @@ getph1byaddr(local, remote)
  * with phase 2's destinaion.
  */
 struct ph1handle *
-getph1bydstaddr(remote)
+getph1bydstaddrwop(remote)
 	struct sockaddr *remote;
 {
 	struct ph1handle *p;
@@ -267,6 +284,11 @@ delph1(iph1)
 		iph1->local = NULL;
 	}
 
+	if (iph1->approval) {
+		delisakmpsa(iph1->approval);
+		iph1->approval = NULL;
+	}
+
 #ifdef ENABLE_HYBRID
 	if (iph1->mode_cfg)
 		isakmp_cfg_rmstate(iph1);
@@ -431,6 +453,38 @@ getph2bymsgid(iph1, msgid)
 	return NULL;
 }
 
+struct ph2handle *
+getph2byid(src, dst, spid)
+	struct sockaddr *src, *dst;
+	u_int32_t spid;
+{
+	struct ph2handle *p;
+
+	LIST_FOREACH(p, &ph2tree, chain) {
+		if (spid == p->spid &&
+		    CMPSADDR(src, p->src) == 0 &&
+		    CMPSADDR(dst, p->dst) == 0)
+			return p;
+	}
+
+	return NULL;
+}
+
+struct ph2handle *
+getph2bysaddr(src, dst)
+	struct sockaddr *src, *dst;
+{
+	struct ph2handle *p;
+
+	LIST_FOREACH(p, &ph2tree, chain) {
+		if (cmpsaddrstrict(src, p->src) == 0 &&
+		    cmpsaddrstrict(dst, p->dst) == 0)
+			return p;
+	}
+
+	return NULL;
+}
+
 /*
  * call by pk_recvexpire().
  */
@@ -615,6 +669,7 @@ flushph2()
 		if (p->status == PHASE2ST_ESTABLISHED) 
 			isakmp_info_send_d2(p);
 
+		delete_spd(p);
 		unbindph12(p);
 		remph2(p);
 		delph2(p);
