@@ -1,4 +1,4 @@
-/* $Id: ipsec_doi.c,v 1.24.2.5 2005/04/25 22:06:32 manubsd Exp $ */
+/* $Id: ipsec_doi.c,v 1.26.2.8 2005/05/31 06:05:53 monas Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -90,6 +90,7 @@ static int switch_authmethod(int);
 #endif
 
 #ifdef HAVE_GSSAPI
+#include <iconv.h>
 #include "gssapi.h"
 #endif
 
@@ -212,19 +213,23 @@ get_ph1approval(iph1, pair)
 	int prophlen;
 	int i;
 
-	iph1->approval = NULL;
+	if (iph1->approval) {
+		delisakmpsa(iph1->approval);
+		iph1->approval = NULL;
+	}
 
 	for (i = 0; i < MAXPROPPAIRLEN; i++) {
 		if (pair[i] == NULL)
 			continue;
 		for (s = pair[i]; s; s = s->next) {
-			prophlen = sizeof(struct isakmp_pl_p)
-					+ s->prop->spi_size;
+			prophlen = 
+			    sizeof(struct isakmp_pl_p) + s->prop->spi_size;
+
 			/* compare proposal and select one */
 			for (p = s; p; p = p->tnext) {
-				sa = get_ph1approvalx(p, iph1->rmconf->proposal,
-						      &tsa, iph1->rmconf->pcheck_level);
-				if (sa != NULL)
+				if ((sa = get_ph1approvalx(p, 
+				    iph1->rmconf->proposal, &tsa, 
+				    iph1->rmconf->pcheck_level)) != NULL)
 					goto found;
 			}
 		}
@@ -274,8 +279,8 @@ found:
 saok:
 #ifdef HAVE_GSSAPI
 	if (sa->gssid != NULL)
-		plog(LLV_DEBUG, LOCATION, NULL, "gss id in new sa '%s'\n",
-		    sa->gssid->v);
+		plog(LLV_DEBUG, LOCATION, NULL, "gss id in new sa '%.*s'\n",
+		    sa->gssid->l, sa->gssid->v);
 	if (iph1-> side == INITIATOR) {
 		if (iph1->rmconf->proposal->gssid != NULL)
 			iph1->gi_i = vdup(iph1->rmconf->proposal->gssid);
@@ -285,29 +290,27 @@ saok:
 	} else {
 		if (tsa.gssid != NULL) {
 			iph1->gi_r = vdup(tsa.gssid);
-			if (iph1->rmconf->proposal->gssid != NULL)
-				iph1->gi_i =
-				    vdup(iph1->rmconf->proposal->gssid);
-			else
-				iph1->gi_i = gssapi_get_default_id(iph1);
+			iph1->gi_i = gssapi_get_id(iph1);
 			if (sa->gssid == NULL && iph1->gi_i != NULL)
 				sa->gssid = vdup(iph1->gi_i);
 		}
 		iph1->approval = sa;
 	}
 	if (iph1->gi_i != NULL)
-		plog(LLV_DEBUG, LOCATION, NULL, "GIi is %*s\n",
+		plog(LLV_DEBUG, LOCATION, NULL, "GIi is %.*s\n",
 		    iph1->gi_i->l, iph1->gi_i->v);
 	if (iph1->gi_r != NULL)
-		plog(LLV_DEBUG, LOCATION, NULL, "GIr is %*s\n",
+		plog(LLV_DEBUG, LOCATION, NULL, "GIr is %.*s\n",
 		    iph1->gi_r->l, iph1->gi_r->v);
 #else
 	iph1->approval = sa;
 #endif
 
 	newsa = get_sabyproppair(p, iph1);
-	if (newsa == NULL)
+	if (newsa == NULL) {
+		delisakmpsa(iph1->approval);
 		iph1->approval = NULL;
+	}
 
 	return newsa;
 }
@@ -389,32 +392,71 @@ get_ph1approvalx(p, proposal, sap, check_level)
 		    tsap->authmethod == authmethod &&
 		    tsap->hashtype == s->hashtype &&
 		    tsap->dh_group == s->dh_group &&
-		    tsap->encklen == s->encklen)
-			switch(check_level){
+		    tsap->encklen == s->encklen) {
+			switch(check_level) {
 			case PROP_CHECK_OBEY:
-				if (s->rmconf && s->rmconf->remote->sa_family != AF_UNSPEC)
-					s->lifetime=tsap->lifetime;
 				goto found;
 				break;
+
 			case PROP_CHECK_STRICT:
-			case PROP_CHECK_CLAIM:
-				if (tsap->lifetime > s->lifetime) 
-					continue ;
-				if (s->rmconf && s->rmconf->remote->sa_family != AF_UNSPEC)
-					s->lifetime=tsap->lifetime;
+				if ((tsap->lifetime > s->lifetime) ||
+				    (tsap->lifebyte > s->lifebyte))
+					continue;
 				goto found;
 				break;
-			case PROP_CHECK_EXACT:
-				if (tsap->lifetime != s->lifetime) 
-					continue ;
+
+			case PROP_CHECK_CLAIM:
+				if (tsap->lifetime < s->lifetime)
+					s->lifetime = tsap->lifetime;
+				if (tsap->lifebyte < s->lifebyte)
+					s->lifebyte = tsap->lifebyte;
 				goto found;
 				break;
+
+			case PROP_CHECK_EXACT:
+				if ((tsap->lifetime != s->lifetime) ||
+				    (tsap->lifebyte != s->lifebyte))
+					continue;
+				goto found;
+				break;
+
+			default:
+				plog(LLV_ERROR, LOCATION, NULL, 
+				    "Unexpected proposal_check value\n");
+				continue;
+				break;
 			}
+		}
 	}
 
 found:
 	if (tsap->dhgrp != NULL)
 		oakley_dhgrp_free(tsap->dhgrp);
+
+	if ((s = dupisakmpsa(s)) != NULL) {
+		switch(check_level) {
+		case PROP_CHECK_OBEY:
+			s->lifetime = tsap->lifetime;
+			s->lifebyte = tsap->lifebyte;
+			break;
+
+		case PROP_CHECK_STRICT:
+			s->lifetime = tsap->lifetime;
+			s->lifebyte = tsap->lifebyte;
+			break;
+
+		case PROP_CHECK_CLAIM:
+			if (tsap->lifetime < s->lifetime)
+				s->lifetime = tsap->lifetime;
+			if (tsap->lifebyte < s->lifebyte)
+				s->lifebyte = tsap->lifebyte;
+			break;
+
+		default:
+			break;
+		}
+	}
+
 	return s;
 }
 
@@ -693,16 +735,81 @@ t2isakmpsa(trns, sa)
 #ifdef HAVE_GSSAPI
 		case OAKLEY_ATTR_GSS_ID:
 		{
+			iconv_t cd;
+			size_t srcleft, dstleft, rv;
+			const char *src;
+			char *dst;
 			int len = ntohs(d->lorv);
 
-			sa->gssid = vmalloc(len);
-			memcpy(sa->gssid->v, d + 1, len);
+			/*
+			 * Older verions of racoon just placed the
+			 * ISO-Latin-1 string on the wire directly.
+			 * Check to see if we are configured to be
+			 * compatible with this behavior.
+			 */
+			if (lcconf->gss_id_enc == LC_GSSENC_LATIN1) {
+				sa->gssid = vmalloc(len);
+				memcpy(sa->gssid->v, d + 1, len);
+				plog(LLV_DEBUG, LOCATION, NULL,
+				  "received old-style gss id '%.*s' (len %d)\n",
+				  sa->gssid->l, sa->gssid->v, sa->gssid->l);
+				break;
+			}
+
+			/*
+			 * For Windows 2000 compatibility, we expect
+			 * the GSS ID attribute on the wire to be
+			 * encoded in UTF-16LE.  Internally, we work
+			 * in ISO-Latin-1.  Therefore, we should need
+			 * 1/2 the specified length, which should always
+			 * be a multiple of 2 octets.
+			 */
+			cd = iconv_open("latin1", "utf-16le");
+			if (cd == (iconv_t) -1) {
+				plog(LLV_ERROR, LOCATION, NULL,
+				    "unable to initialize utf-16le -> latin1 "
+				    "conversion descriptor: %s\n",
+				    strerror(errno));
+				break;
+			}
+
+			sa->gssid = vmalloc(len / 2);
+
+			src = (const char *)(d + 1);
+			srcleft = len;
+
+			dst = sa->gssid->v;
+			dstleft = len / 2;
+
+			rv = iconv(cd, &src, &srcleft, &dst, &dstleft);
+			if (rv != 0) {
+				if (rv == -1) {
+					plog(LLV_ERROR, LOCATION, NULL,
+					    "unable to convert GSS ID from "
+					    "utf-16le -> latin1: %s\n",
+					    strerror(errno));
+				} else {
+					plog(LLV_ERROR, LOCATION, NULL,
+					    "%zd character%s in GSS ID cannot "
+					    "be represented in latin1\n",
+					    rv, rv == 1 ? "" : "s");
+				}
+				(void) iconv_close(cd);
+				vfree(sa->gssid);
+				sa->gssid = NULL;
+				break;
+			}
+			(void) iconv_close(cd);
+
+			/* XXX dstleft should always be 0; assert it? */
+			sa->gssid->l = (len / 2) - dstleft;
+
 			plog(LLV_DEBUG, LOCATION, NULL,
-			    "received gss id '%s' (len %d)\n", sa->gssid->v,
-			    sa->gssid->l);
+			    "received gss id '%.*s' (len %d)\n",
+			    sa->gssid->l, sa->gssid->v, sa->gssid->l);
 			break;
 		}
-#endif
+#endif /* HAVE_GSSAPI */
 
 		default:
 			break;
@@ -2325,6 +2432,15 @@ check_attr_ipcomp(trns)
 			case IPSECDOI_ATTR_ENC_MODE_TUNNEL:
 			case IPSECDOI_ATTR_ENC_MODE_TRNS:
 				break;
+#ifdef ENABLE_NATT
+			case IPSECDOI_ATTR_ENC_MODE_UDPTUNNEL_RFC:
+			case IPSECDOI_ATTR_ENC_MODE_UDPTRNS_RFC:
+			case IPSECDOI_ATTR_ENC_MODE_UDPTUNNEL_DRAFT:
+			case IPSECDOI_ATTR_ENC_MODE_UDPTRNS_DRAFT:
+				plog(LLV_DEBUG, LOCATION, NULL,
+				     "UDP encapsulation requested\n");
+				break;
+#endif
 			default:
 				plog(LLV_ERROR, LOCATION, NULL,
 					"invalid encryption mode=%u.\n",
@@ -2649,16 +2765,77 @@ setph1attr(sa, buf)
 	if (sa->authmethod == OAKLEY_ATTR_AUTH_METHOD_GSSAPI_KRB &&
 	    sa->gssid != NULL) {
 		attrlen += sizeof(struct isakmp_data);
-		attrlen += sa->gssid->l;
+		/*
+		 * Older versions of racoon just placed the ISO-Latin-1
+		 * string on the wire directly.  Check to see if we are
+		 * configured to be compatible with this behavior.  Otherwise,
+		 * we encode the GSS ID as UTF-16LE for Windows 2000
+		 * compatibility, which requires twice the number of octets.
+		 */
+		if (lcconf->gss_id_enc == LC_GSSENC_LATIN1)
+			attrlen += sa->gssid->l;
+		else
+			attrlen += sa->gssid->l * 2;
 		if (buf) {
 			plog(LLV_DEBUG, LOCATION, NULL, "gss id attr: len %d, "
-			    "val '%s'\n", sa->gssid->l, sa->gssid->v);
-			p = isakmp_set_attr_v(p, OAKLEY_ATTR_GSS_ID,
-				(caddr_t)sa->gssid->v, 
-				sa->gssid->l);
+			    "val '%.*s'\n", sa->gssid->l, sa->gssid->l,
+			    sa->gssid->v);
+			if (lcconf->gss_id_enc == LC_GSSENC_LATIN1) {
+				p = isakmp_set_attr_v(p, OAKLEY_ATTR_GSS_ID,
+					(caddr_t)sa->gssid->v,
+					sa->gssid->l);
+			} else {
+				size_t dstleft = sa->gssid->l * 2;
+				size_t srcleft = sa->gssid->l;
+				const char *src = (const char *)sa->gssid->v;
+				char *odst, *dst = racoon_malloc(dstleft);
+				iconv_t cd;
+				size_t rv;
+
+				cd = iconv_open("utf-16le", "latin1");
+				if (cd == (iconv_t) -1) {
+					plog(LLV_ERROR, LOCATION, NULL,
+					    "unable to initialize "
+					    "latin1 -> utf-16le "
+					    "converstion descriptor: %s\n",
+					    strerror(errno));
+					attrlen -= sa->gssid->l * 2;
+					goto gssid_done;
+				}
+				odst = dst;
+				rv = iconv(cd, &src, &srcleft,
+				    &dst, &dstleft);
+				if (rv != 0) {
+					if (rv == -1) {
+						plog(LLV_ERROR, LOCATION, NULL,
+						    "unable to convert GSS ID "
+						    "from latin1 -> utf-16le: "
+						    "%s\n", strerror(errno));
+					} else {
+						/* should never happen */
+						plog(LLV_ERROR, LOCATION, NULL,
+						    "%zd character%s in GSS ID "
+						    "cannot be represented "
+						    "in utf-16le\n",
+						    rv, rv == 1 ? "" : "s");
+					}
+					(void) iconv_close(cd);
+					attrlen -= sa->gssid->l * 2;
+					goto gssid_done;
+				}
+				(void) iconv_close(cd);
+
+				/* XXX Check srcleft and dstleft? */
+
+				p = isakmp_set_attr_v(p, OAKLEY_ATTR_GSS_ID,
+					odst, sa->gssid->l * 2);
+
+				racoon_free(odst);
+			}
 		}
 	}
-#endif
+ gssid_done:
+#endif /* HAVE_GSSAPI */
 
 	return attrlen;
 }
@@ -3980,36 +4157,23 @@ struct isakmpsa *
 fixup_initiator_sa(match, received)
 	struct isakmpsa *match, *received;
 {
-	struct isakmpsa *newsa;
+	if (received->gssid != NULL)
+		match->gssid = vdup(received->gssid);
 
-	if (received->gssid == NULL)
-		return match;
-
-	newsa = newisakmpsa();
-	memcpy(newsa, match, sizeof *newsa);
-
-	if (match->dhgrp != NULL) {
-		newsa->dhgrp = racoon_calloc(1, sizeof(struct dhgroup));
-		memcpy(newsa->dhgrp, match->dhgrp, sizeof (struct dhgroup));
-	}
-	newsa->next = NULL;
-	newsa->rmconf = NULL;
-
-	newsa->gssid = vdup(received->gssid);
-
-	return newsa;
+	return match;
 }
 #endif
 
 static int rm_idtype2doi[] = {
-	IPSECDOI_ID_FQDN,
-	IPSECDOI_ID_USER_FQDN,
-	IPSECDOI_ID_KEY_ID,
-	255,	/* it's type of "address"
+	255,				/* IDTYPE_UNDEFINED, 0 */
+	IPSECDOI_ID_FQDN,		/* IDTYPE_FQDN, 1 */
+	IPSECDOI_ID_USER_FQDN,		/* IDTYPE_USERFQDN, 2 */
+	IPSECDOI_ID_KEY_ID,		/* IDTYPE_KEYID, 3 */
+	255,    /*			   IDTYPE_ADDRESS, 4 
 		 * it expands into 4 types by another function. */
-	IPSECDOI_ID_DER_ASN1_DN,
+	IPSECDOI_ID_DER_ASN1_DN,	/* IDTYPE_ASN1DN, 5 */
 #ifdef ENABLE_HYBRID
-	255,	/* It's type LOGIN */
+	255,				/* IDTYPE_LOGIN, 6 */
 #endif
 };
 
