@@ -1,4 +1,4 @@
-/* $Id: isakmp.c,v 1.32.2.4 2005/04/21 08:57:17 monas Exp $ */
+/* $Id: isakmp.c,v 1.34.2.13 2005/06/22 21:20:35 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -39,6 +39,12 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
+#ifndef HAVE_NETINET6_IPSEC
+#include <netinet/ipsec.h>
+#else 
+#include <netinet6/ipsec.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -58,6 +64,7 @@
 #include <unistd.h>
 #endif
 #include <ctype.h>
+#include <fcntl.h>
 
 #include "var.h"
 #include "misc.h"
@@ -70,11 +77,14 @@
 #include "remoteconf.h"
 #include "localconf.h"
 #include "grabmyaddr.h"
+#include "admin.h"
+#include "privsep.h"
 #include "isakmp_var.h"
 #include "isakmp.h"
 #include "oakley.h"
 #include "evt.h"
 #include "handler.h"
+#include "proposal.h"
 #include "ipsec_doi.h"
 #include "pfkey.h"
 #include "crypto_openssl.h"
@@ -98,6 +108,8 @@
 # include "nattraversal.h"
 # ifdef __linux__
 #  include <linux/udp.h>
+#include <fcntl.h>
+
 #  ifndef SOL_UDP
 #   define SOL_UDP 17
 #  endif
@@ -170,8 +182,6 @@ static int frag_handler(struct ph1handle *,
     vchar_t *, struct sockaddr *, struct sockaddr *);
 #endif
 
-extern char **environ;
-
 /*
  * isakmp packet handler
  */
@@ -206,8 +216,16 @@ isakmp_handler(so_isakmp)
 	}
 
 	/* keep-alive packet - ignore */
-	if (len == 1 && (x.buf[0]&0xff) == 0xff)
+	if (len == 1 && (x.buf[0]&0xff) == 0xff) {
+		/* Pull the keep-alive packet */
+		if ((len = recvfrom(so_isakmp, (char *)x.buf, 1,
+		    0, (struct sockaddr *)&remote, &remote_len)) != 1) {
+			plog(LLV_ERROR, LOCATION, NULL,
+			    "failed to receive keep alive packet: %s\n",
+			    strerror (errno));
+		}
 		goto end;
+	}
 
 #ifdef ENABLE_NATT
 	/* we don't know about portchange yet, 
@@ -429,14 +447,14 @@ isakmp_main(msg, remote, local)
 
 #ifdef ENABLE_NATT
 		/* Floating ports for NAT-T */
-		if ((iph1->natt_flags & NAT_DETECTED) &&
+		if (NATT_AVAILABLE(iph1) &&
 		    ! (iph1->natt_flags & NAT_PORTS_CHANGED) &&
 		    ((cmpsaddrstrict(iph1->remote, remote) != 0) ||
 		    (cmpsaddrstrict(iph1->local, local) != 0)))
 		{
 			/* prevent memory leak */
-			racoon_free (iph1->remote);
-			racoon_free (iph1->local);
+			racoon_free(iph1->remote);
+			racoon_free(iph1->local);
 
 			/* copy-in new addresses */
 			iph1->remote = dupsaddr(remote);
@@ -445,7 +463,7 @@ isakmp_main(msg, remote, local)
 			/* set the flag to prevent further port floating
 			   (FIXME: should we allow it? E.g. when the NAT gw 
 			    is rebooted?) */
-			iph1->natt_flags |= NAT_PORTS_CHANGED;
+			iph1->natt_flags |= NAT_PORTS_CHANGED | NAT_ADD_NON_ESP_MARKER;
 			
 			/* print some neat info */
 			plog (LLV_INFO, LOCATION, NULL, 
@@ -1156,7 +1174,6 @@ isakmp_ph2begin_i(iph1, iph2)
 		unbindph12(iph2);
 		/* release ipsecsa handler due to internal error. */
 		remph2(iph2);
-		delph2(iph2);
 		return -1;
 	}
 	return 0;
@@ -1207,11 +1224,15 @@ isakmp_ph2begin_r(iph1, msg)
 	}
 	switch (iph2->dst->sa_family) {
 	case AF_INET:
+#ifndef ENABLE_NATT
 		((struct sockaddr_in *)iph2->dst)->sin_port = 0;
+#endif
 		break;
 #ifdef INET6
 	case AF_INET6:
+#ifndef ENABLE_NATT
 		((struct sockaddr_in6 *)iph2->dst)->sin6_port = 0;
+#endif
 		break;
 #endif
 	default:
@@ -1228,11 +1249,15 @@ isakmp_ph2begin_r(iph1, msg)
 	}
 	switch (iph2->src->sa_family) {
 	case AF_INET:
+#ifndef ENABLE_NATT
 		((struct sockaddr_in *)iph2->src)->sin_port = 0;
+#endif
 		break;
 #ifdef INET6
 	case AF_INET6:
+#ifndef ENABLE_NATT
 		((struct sockaddr_in6 *)iph2->src)->sin6_port = 0;
+#endif
 		break;
 #endif
 	default:
@@ -1550,6 +1575,10 @@ isakmp_open()
 				plog(LLV_ERROR, LOCATION, NULL,
 					"setsockopt(%d): %s\n",
 					pktinfo, strerror(errno));
+		if (fcntl(p->sock, F_SETFL, O_NONBLOCK) == -1)
+			plog(LLV_WARNING, LOCATION, NULL,
+				"failed to put socket in non-blocking mode\n");
+
 				goto err_and_next;
 			}
 			break;
@@ -1597,11 +1626,11 @@ isakmp_open()
 			if(option != -1){
 				if (setsockopt (p->sock, SOL_UDP, UDP_ENCAP,
 								&option, sizeof (option)) < 0) {
-					plog(LLV_ERROR, LOCATION, NULL,
+					plog(LLV_WARNING, LOCATION, NULL,
 						 "setsockopt(%s): %s\n",
 						 option == UDP_ENCAP_ESPINUDP ? "UDP_ENCAP_ESPINUDP" : "UDP_ENCAP_ESPINUDP_NON_IKE",
 						 strerror(errno));
-					goto err_and_next;
+					goto skip_encap;
 				}
 				else {
 					plog(LLV_INFO, LOCATION, NULL,
@@ -1611,8 +1640,8 @@ isakmp_open()
 				}
 			}
 		}
+skip_encap:
 #endif
-
 		continue;
 
 	err_and_next:
@@ -1671,7 +1700,7 @@ isakmp_send(iph1, sbuf)
 	vchar_t *vbuf = NULL;
 
 #ifdef ENABLE_NATT
-	size_t extralen = NON_ESP_MARKER_USE(iph1) * NON_ESP_MARKER_LEN;
+	size_t extralen = NON_ESP_MARKER_USE(iph1) ? NON_ESP_MARKER_LEN : 0;
 
 #ifdef ENABLE_FRAG
 	/* 
@@ -1984,8 +2013,23 @@ isakmp_post_acquire(iph2)
 		return 0;
 	}
 
-	/* search isakmp status table by address with masking port */
+	/* 
+	 * Search isakmp status table by address and port 
+	 * If NAT-T is in use, consider null ports as a 
+	 * wildcard and use IKE ports instead.
+	 */
+#ifdef ENABLE_NATT
+	if (!extract_port(iph2->src) && !extract_port(iph2->dst)) {
+		if ((iph1 = getph1byaddrwop(iph2->src, iph2->dst)) != NULL) {
+			set_port(iph2->src, extract_port(iph1->local));
+			set_port(iph2->dst, extract_port(iph1->remote));
+		}
+	} else {
+		iph1 = getph1byaddr(iph2->src, iph2->dst);
+	}
+#else
 	iph1 = getph1byaddr(iph2->src, iph2->dst);
+#endif
 
 	/* no ISAKMP-SA found. */
 	if (iph1 == NULL) {
@@ -2103,7 +2147,23 @@ isakmp_chkph1there(iph2)
 		return;
 	}
 
+	/* 
+	 * Search isakmp status table by address and port 
+	 * If NAT-T is in use, consider null ports as a 
+	 * wildcard and use IKE ports instead.
+	 */
+#ifdef ENABLE_NATT
+	if (!extract_port(iph2->src) && !extract_port(iph2->dst)) {
+		if ((iph1 = getph1byaddrwop(iph2->src, iph2->dst)) != NULL) {
+			set_port(iph2->src, extract_port(iph1->local));
+			set_port(iph2->dst, extract_port(iph1->remote));
+		}
+	} else {
+		iph1 = getph1byaddr(iph2->src, iph2->dst);
+	}
+#else
 	iph1 = getph1byaddr(iph2->src, iph2->dst);
+#endif
 
 	/* XXX Even if ph1 as responder is there, should we not start
 	 * phase 2 negotiation ? */
@@ -2675,6 +2735,7 @@ copy_ph1addresses(iph1, rmconf, remote, local)
 		delph1(iph1);
 		return -1;
 	}
+	port = NULL;
 	switch (iph1->local->sa_family) {
 	case AF_INET:
 		port = &((struct sockaddr_in *)iph1->local)->sin_port;
@@ -2702,6 +2763,12 @@ copy_ph1addresses(iph1, rmconf, remote, local)
 		delph1(iph1);
 		return -1;
 	}
+#ifdef ENABLE_NATT
+	if ( port != NULL && *port == htons(lcconf->port_isakmp_natt) ) {
+	    plog (LLV_DEBUG, LOCATION, NULL, "Marking ports as changed\n");
+	    iph1->natt_flags |= NAT_ADD_NON_ESP_MARKER;
+	}
+#endif
 
 	return 0;
 }
@@ -2842,47 +2909,505 @@ script_hook(iph1, script)
 	int script;
 {
 #define IP_MAX 40
+#define PORT_MAX 6
 	char addrstr[IP_MAX];
-	char *argv[3];
+	char portstr[PORT_MAX];
+	char **envp = NULL;
+	int envc = 1;
 	struct sockaddr_in *sin;
+	char **c;
 
-	if (iph1->rmconf->script[script] == NULL)
+	if (iph1->rmconf->script[script] == -1)
 		return;
 
 #ifdef ENABLE_HYBRID
-	isakmp_cfg_setenv(iph1);
+	(void)isakmp_cfg_setenv(iph1, &envp, &envc);
 #endif
 
 	/* local address */
 	sin = (struct sockaddr_in *)iph1->local;
 	inet_ntop(sin->sin_family, &sin->sin_addr, addrstr, IP_MAX);
-	setenv("LOCAL_ADDR", addrstr, 1);
+	snprintf(portstr, PORT_MAX, "%d", ntohs(sin->sin_port));
 
+	if (script_env_append(&envp, &envc, "LOCAL_ADDR", addrstr) != 0) {
+		plog(LLV_ERROR, LOCATION, NULL, "Cannot set LOCAL_ADDR\n");
+		goto out;
+	}
+
+	if (script_env_append(&envp, &envc, "LOCAL_PORT", portstr) != 0) {
+		plog(LLV_ERROR, LOCATION, NULL, "Cannot set LOCAL_PORT\n");
+		goto out;
+	}
 
 	/* Peer address */
 	sin = (struct sockaddr_in *)iph1->remote;
 	inet_ntop(sin->sin_family, &sin->sin_addr, addrstr, IP_MAX);
-	setenv("REMOTE_ADDR", addrstr, 1);
+	snprintf(portstr, PORT_MAX, "%d", ntohs(sin->sin_port));
 
-	argv[0] = iph1->rmconf->script[script]->v;
-	argv[1] = script_names[script];
-	argv[2] = NULL;
-
-	switch (fork()) {
-	case 0:
-		execve(argv[0], argv, environ);
-		plog(LLV_ERROR, LOCATION, NULL, "execve(\"%s\") failed: %s\n",
-		    argv[0], strerror(errno));
-		return;
-		break;
-	case -1:
-		plog(LLV_ERROR, LOCATION, NULL, "Cannot fork: %s\n",
-		    strerror(errno));
-		return;
-		break;
-	default:
-		break;
+	if (script_env_append(&envp, &envc, "REMOTE_ADDR", addrstr) != 0) {
+		plog(LLV_ERROR, LOCATION, NULL, "Cannot set REMOTE_ADDR\n");
+		goto out;
 	}
 
+	if (script_env_append(&envp, &envc, "REMOTE_PORT", portstr) != 0) {
+		plog(LLV_ERROR, LOCATION, NULL, "Cannot set REMOTEL_PORT\n");
+		goto out;
+	}
+
+	if (privsep_script_exec(iph1->rmconf->script[script], 
+	    script, envp) != 0) 
+		plog(LLV_ERROR, LOCATION, NULL, 
+		    "Script %s execution failed\n", script_names[script]);
+
+out:
+	for (c = envp; *c; c++)
+		racoon_free(*c);
+
+	racoon_free(envp);
+
 	return;
 }
+
+int
+script_env_append(envp, envc, name, value)
+	char ***envp;
+	int *envc;
+	char *name;
+	char *value;
+{
+	char *envitem;
+	char **newenvp;
+	int newenvc;
+
+	envitem = racoon_malloc(strlen(name) + 1 + strlen(value) + 1);
+	if (envitem == NULL) {
+		plog(LLV_ERROR, LOCATION, NULL,
+		    "Cannot allocate memory: %s\n", strerror(errno));
+		return -1;
+	}
+	sprintf(envitem, "%s=%s", name, value);
+
+	newenvc = (*envc) + 1;
+	newenvp = racoon_realloc(*envp, newenvc * sizeof(char *));
+	if (newenvp == NULL) {
+		plog(LLV_ERROR, LOCATION, NULL,
+		    "Cannot allocate memory: %s\n", strerror(errno));
+		return -1;
+	}
+
+	newenvp[newenvc - 2] = envitem;
+	newenvp[newenvc - 1] = NULL;
+
+	*envp = newenvp;
+	*envc = newenvc;
+	return 0;
+}
+
+int
+script_exec(script, name, envp)
+	int script;
+	int name;
+	char *const envp[];
+{
+	char *argv[] = { NULL, NULL, NULL };
+	vchar_t **sp;
+
+	if (script_paths == NULL) {
+		plog(LLV_ERROR, LOCATION, NULL,
+		    "privsep_script_exec: script_paths was not initialized\n");
+		return -1;
+	}
+
+	sp = (vchar_t **)(script_paths->v);
+
+	argv[0] = sp[script]->v;
+	argv[1] = script_names[name];
+	argv[2] = NULL;
+
+	switch (fork()) { 
+	case 0:
+		execve(argv[0], argv, envp);
+		plog(LLV_ERROR, LOCATION, NULL,
+		    "execve(\"%s\") failed: %s\n",
+		    argv[0], strerror(errno));
+		_exit(1);
+		break;
+	case -1:
+		plog(LLV_ERROR, LOCATION, NULL,
+		    "Cannot fork: %s\n", strerror(errno));
+		return -1;
+		break;
+	default:
+		break;
+	}	
+
+	return 0;
+}
+
+void
+purge_remote(iph1)
+	struct ph1handle *iph1;
+{
+	vchar_t *buf = NULL;
+	struct sadb_msg *msg, *next, *end;
+	struct sadb_sa *sa;
+	struct sockaddr *src, *dst;
+	caddr_t mhp[SADB_EXT_MAX + 1];
+	struct ph2handle *iph2;
+
+	/* Delete all phase2 SAs */
+	buf = pfkey_dump_sadb(SADB_SATYPE_UNSPEC);
+	if (buf == NULL) {
+		plog(LLV_DEBUG, LOCATION, NULL,
+			"pfkey_dump_sadb returned nothing.\n");
+		return;
+	}
+
+	msg = (struct sadb_msg *)buf->v;
+	end = (struct sadb_msg *)(buf->v + buf->l);
+
+	while (msg < end) {
+		if ((msg->sadb_msg_len << 3) < sizeof(*msg))
+			break;
+		next = (struct sadb_msg *)((caddr_t)msg + (msg->sadb_msg_len << 3));
+		if (msg->sadb_msg_type != SADB_DUMP) {
+			msg = next;
+			continue;
+		}
+
+		if (pfkey_align(msg, mhp) || pfkey_check(mhp)) {
+			plog(LLV_ERROR, LOCATION, NULL,
+				"pfkey_check (%s)\n", ipsec_strerror());
+			msg = next;
+			continue;
+		}
+
+		sa = (struct sadb_sa *)(mhp[SADB_EXT_SA]);
+		if (!sa ||
+		    !mhp[SADB_EXT_ADDRESS_SRC] ||
+		    !mhp[SADB_EXT_ADDRESS_DST]) {
+			msg = next;
+			continue;
+		}
+		src = PFKEY_ADDR_SADDR(mhp[SADB_EXT_ADDRESS_SRC]);
+		dst = PFKEY_ADDR_SADDR(mhp[SADB_EXT_ADDRESS_DST]);
+
+		if (sa->sadb_sa_state != SADB_SASTATE_MATURE &&
+		    sa->sadb_sa_state != SADB_SASTATE_DYING) {
+			msg = next;
+			continue;
+		}
+
+		/* delete in/outbound SAs */
+		if (CMPSADDR(iph1->remote, dst) &&
+		    CMPSADDR(iph1->remote, src)) {
+			msg = next;
+			continue;
+		}
+
+		pfkey_send_delete(lcconf->sock_pfkey,
+				  msg->sadb_msg_satype,
+				  IPSEC_MODE_ANY,
+				  src, dst, sa->sadb_sa_spi);
+
+		/*
+		 * delete a relative phase 2 handler.
+		 * continue to process if no relative phase 2 handler
+		 * exists.
+		 */
+		while ((iph2 = getph2bysaddr(src, dst)) != NULL) {
+			delete_spd(iph2);
+			unbindph12(iph2);
+			remph2(iph2);
+			delph2(iph2);
+		}
+		
+		plog(LLV_INFO, LOCATION, NULL,
+			 "purged IPsec-SA spi=%u.\n",
+			 ntohl(sa->sadb_sa_spi));
+
+		msg = next;
+	}
+
+	if (buf)
+		vfree(buf);
+
+	/* Mark the phase1 handler as EXPIRED */
+	plog(LLV_INFO, LOCATION, NULL,
+		 "purged ISAKMP-SA spi=%s.\n",
+		 isakmp_pindex(&(iph1->index), iph1->msgid));
+
+	if (iph1->sce)
+		SCHED_KILL(iph1->sce);
+
+	iph1->status = PHASE1ST_EXPIRED;
+	iph1->sce = sched_new(1, isakmp_ph1delete_stub, iph1);
+}
+
+void 
+delete_spd(iph2)
+	struct ph2handle *iph2;
+{
+	if (iph2 == NULL)
+		return;
+
+	/* Delete the SPD entry if we generated it
+	 */
+	if (iph2->generated_spidx) {
+		struct policyindex spidx;
+		struct sockaddr_storage addr;
+		u_int8_t pref;
+		struct sockaddr *src = iph2->src;
+		struct sockaddr *dst = iph2->dst;
+		int error;
+		int idi2type = 0;/* switch whether copy IDs into id[src,dst]. */
+
+		plog(LLV_INFO, LOCATION, NULL,
+			 "generated policy, deleting it.\n");
+		
+		memset(&spidx, 0, sizeof(spidx));
+		iph2->spidx_gen = (caddr_t )&spidx;
+		
+		/* make inbound policy */
+		iph2->src = dst;
+		iph2->dst = src;
+		spidx.dir = IPSEC_DIR_INBOUND;
+		spidx.ul_proto = 0;
+		
+		/* 
+		 * Note: code from get_proposal_r
+		 */
+		
+#define _XIDT(d) ((struct ipsecdoi_id_b *)(d)->v)->type
+		
+		/*
+		 * make destination address in spidx from either ID payload
+		 * or phase 1 address into a address in spidx.
+		 */
+		if (iph2->id != NULL
+			&& (_XIDT(iph2->id) == IPSECDOI_ID_IPV4_ADDR
+			|| _XIDT(iph2->id) == IPSECDOI_ID_IPV6_ADDR
+			|| _XIDT(iph2->id) == IPSECDOI_ID_IPV4_ADDR_SUBNET
+			|| _XIDT(iph2->id) == IPSECDOI_ID_IPV6_ADDR_SUBNET)) {
+			/* get a destination address of a policy */
+			error = ipsecdoi_id2sockaddr(iph2->id,
+			    (struct sockaddr *)&spidx.dst,
+			    &spidx.prefd, &spidx.ul_proto);
+			if (error)
+				goto purge;
+			
+#ifdef INET6
+			/*
+			 * get scopeid from the SA address.
+			 * note that the phase 1 source address is used as
+			 * a destination address to search for a inbound 
+			 * policy entry because rcoon is responder.
+			 */
+			if (_XIDT(iph2->id) == IPSECDOI_ID_IPV6_ADDR) {
+				if ((error = 
+				    setscopeid((struct sockaddr *)&spidx.dst,
+				   iph2->src)) != 0)
+					goto purge;
+			}
+#endif
+			
+			if (_XIDT(iph2->id) == IPSECDOI_ID_IPV4_ADDR
+				|| _XIDT(iph2->id) == IPSECDOI_ID_IPV6_ADDR)
+				idi2type = _XIDT(iph2->id);
+			
+		} else {
+			
+			plog(LLV_DEBUG, LOCATION, NULL,
+				 "get a destination address of SP index "
+				 "from phase1 address "
+				 "due to no ID payloads found "
+				 "OR because ID type is not address.\n");
+			
+			/*
+			 * copy the SOURCE address of IKE into the 
+			 * DESTINATION address of the key to search the 
+			 * SPD because the direction of policy is inbound.
+			 */
+			memcpy(&spidx.dst, iph2->src, sysdep_sa_len(iph2->src));
+			switch (spidx.dst.ss_family) {
+				case AF_INET:
+					spidx.prefd = 
+					    sizeof(struct in_addr) << 3;
+					break;
+#ifdef INET6
+				case AF_INET6:
+					spidx.prefd = 
+					    sizeof(struct in6_addr) << 3;
+					break;
+#endif
+				default:
+					spidx.prefd = 0;
+					break;
+			}
+		}
+					
+		/* make source address in spidx */
+		if (iph2->id_p != NULL
+			&& (_XIDT(iph2->id_p) == IPSECDOI_ID_IPV4_ADDR
+			|| _XIDT(iph2->id_p) == IPSECDOI_ID_IPV6_ADDR
+			|| _XIDT(iph2->id_p) == IPSECDOI_ID_IPV4_ADDR_SUBNET
+			|| _XIDT(iph2->id_p) == IPSECDOI_ID_IPV6_ADDR_SUBNET)) {
+			/* get a source address of inbound SA */
+			error = ipsecdoi_id2sockaddr(iph2->id_p,
+			    (struct sockaddr *)&spidx.src,
+			    &spidx.prefs, &spidx.ul_proto);
+			if (error)
+				goto purge;
+
+#ifdef INET6
+			/*
+			 * get scopeid from the SA address.
+			 * for more detail, see above of this function.
+			 */
+			if (_XIDT(iph2->id_p) == IPSECDOI_ID_IPV6_ADDR) {
+				error = 
+				    setscopeid((struct sockaddr *)&spidx.src,
+				    iph2->dst);
+				if (error)
+					goto purge;
+			}
+#endif
+
+			/* make id[src,dst] if both ID types are IP address and same */
+			if (_XIDT(iph2->id_p) == idi2type
+				&& spidx.dst.ss_family == spidx.src.ss_family) {
+				iph2->src_id = 
+				    dupsaddr((struct sockaddr *)&spidx.dst);
+				iph2->dst_id = 
+				    dupsaddr((struct sockaddr *)&spidx.src);
+			}
+
+		} else {
+			plog(LLV_DEBUG, LOCATION, NULL,
+				 "get a source address of SP index "
+				 "from phase1 address "
+				 "due to no ID payloads found "
+				 "OR because ID type is not address.\n");
+
+			/* see above comment. */
+			memcpy(&spidx.src, iph2->dst, sysdep_sa_len(iph2->dst));
+			switch (spidx.src.ss_family) {
+				case AF_INET:
+					spidx.prefs = 
+					    sizeof(struct in_addr) << 3;
+					break;
+#ifdef INET6
+				case AF_INET6:
+					spidx.prefs = 
+					    sizeof(struct in6_addr) << 3;
+					break;
+#endif
+				default:
+					spidx.prefs = 0;
+					break;
+			}
+		}
+
+#undef _XIDT
+
+		plog(LLV_DEBUG, LOCATION, NULL,
+			 "get a src address from ID payload "
+			 "%s prefixlen=%u ul_proto=%u\n",
+			 saddr2str((struct sockaddr *)&spidx.src),
+			 spidx.prefs, spidx.ul_proto);
+		plog(LLV_DEBUG, LOCATION, NULL,
+			 "get dst address from ID payload "
+			 "%s prefixlen=%u ul_proto=%u\n",
+			 saddr2str((struct sockaddr *)&spidx.dst),
+			 spidx.prefd, spidx.ul_proto);
+
+		/*
+		 * convert the ul_proto if it is 0
+		 * because 0 in ID payload means a wild card.
+		 */
+		if (spidx.ul_proto == 0)
+			spidx.ul_proto = IPSEC_ULPROTO_ANY;
+
+#undef _XIDT
+
+		/* End of code from get_proposal_r
+		 */
+
+		if (pk_sendspddelete(iph2) < 0) {
+			plog(LLV_ERROR, LOCATION, NULL,
+				 "pfkey spddelete(inbound) failed.\n");
+		}else{
+			plog(LLV_DEBUG, LOCATION, NULL,
+				 "pfkey spddelete(inbound) sent.\n");
+		}
+
+#ifdef HAVE_POLICY_FWD
+		/* make forward policy if required */
+		if (tunnel_mode_prop(iph2->approval)) {
+			spidx.dir = IPSEC_DIR_FWD;
+			if (pk_sendspddelete(iph2) < 0) {
+				plog(LLV_ERROR, LOCATION, NULL,
+					 "pfkey spddelete(forward) failed.\n");
+			}else{
+				plog(LLV_DEBUG, LOCATION, NULL,
+					 "pfkey spddelete(forward) sent.\n");
+			}
+		}
+#endif
+
+		/* make outbound policy */
+		iph2->src = src;
+		iph2->dst = dst;
+		spidx.dir = IPSEC_DIR_OUTBOUND;
+		addr = spidx.src;
+		spidx.src = spidx.dst;
+		spidx.dst = addr;
+		pref = spidx.prefs;
+		spidx.prefs = spidx.prefd;
+		spidx.prefd = pref;
+
+		if (pk_sendspddelete(iph2) < 0) {
+			plog(LLV_ERROR, LOCATION, NULL,
+				 "pfkey spddelete(outbound) failed.\n");
+		}else{
+			plog(LLV_DEBUG, LOCATION, NULL,
+				 "pfkey spddelete(outbound) sent.\n");
+		}
+purge:
+		iph2->spidx_gen=NULL;
+	}
+}
+
+#ifdef INET6
+u_int32_t
+setscopeid(sp_addr0, sa_addr0)
+	struct sockaddr *sp_addr0, *sa_addr0;
+{
+	struct sockaddr_in6 *sp_addr, *sa_addr;
+    
+	sp_addr = (struct sockaddr_in6 *)sp_addr0;
+	sa_addr = (struct sockaddr_in6 *)sa_addr0;
+
+	if (!IN6_IS_ADDR_LINKLOCAL(&sp_addr->sin6_addr)
+	 && !IN6_IS_ADDR_SITELOCAL(&sp_addr->sin6_addr)
+	 && !IN6_IS_ADDR_MULTICAST(&sp_addr->sin6_addr))
+		return 0;
+
+	/* this check should not be here ? */
+	if (sa_addr->sin6_family != AF_INET6) {
+		plog(LLV_ERROR, LOCATION, NULL,
+			"can't get scope ID: family mismatch\n");
+		return -1;
+	}
+
+	if (!IN6_IS_ADDR_LINKLOCAL(&sa_addr->sin6_addr)) {
+		plog(LLV_ERROR, LOCATION, NULL,
+			"scope ID is not supported except of lladdr.\n");
+		return -1;
+	}
+
+	sp_addr->sin6_scope_id = sa_addr->sin6_scope_id;
+
+	return 0;
+}
+#endif
