@@ -1,4 +1,4 @@
-/* $Id: isakmp_agg.c,v 1.19.2.3 2005/01/26 19:39:26 vanhu Exp $ */
+/* $Id: isakmp_agg.c,v 1.20.2.1 2005/04/09 22:32:06 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -304,6 +304,12 @@ end:
 	if (vid_dpd != NULL)
 		vfree(vid_dpd);
 #endif
+#ifdef ENABLE_HYBRID
+	if (vid_xauth != NULL)
+		vfree(vid_xauth);
+	if (vid_unity != NULL)
+		vfree(vid_unity);
+#endif
 
 	return error;
 }
