@@ -1,4 +1,4 @@
-/* $Id: isakmp_cfg.c,v 1.22.2.1 2005/04/04 21:37:58 manubsd Exp $ */
+/* $Id: isakmp_cfg.c,v 1.26.2.5 2005/05/10 09:45:46 manubsd Exp $ */
 
 /*
  * Copyright (C) 2004 Emmanuel Dreyfus
@@ -84,19 +84,23 @@
 #include "isakmp_unity.h"
 #include "isakmp_cfg.h"
 #include "strnames.h"
+#include "admin.h"
+#include "privsep.h"
 
 struct isakmp_cfg_config isakmp_cfg_config = {
 	0x00000000, 	/* network4 */
 	0x00000000,	/* netmask4 */
 	0x00000000,	/* dns4 */
 	0x00000000,	/* nbns4 */
-	NULL,		/* used4 */
+	NULL,		/* pool */
 	ISAKMP_CFG_AUTH_SYSTEM,		/* authsource */
 	ISAKMP_CFG_CONF_LOCAL,		/* confsource */
 	ISAKMP_CFG_ACCT_NONE,		/* accounting */
 	ISAKMP_CFG_MAX_CNX,		/* pool_size */
 	THROTTLE_PENALTY,		/* auth_throttle */
 	ISAKMP_CFG_MOTD,		/* motd */
+	0,				/* pfs_group */
+	0,				/* save_passwd */
 };
 
 static vchar_t *buffer_cat(vchar_t *s, vchar_t *append);
@@ -111,6 +115,9 @@ static void isakmp_cfg_getaddr4(struct isakmp_data *, struct in_addr *);
 #define ISAKMP_CFG_LOGIN	1
 #define ISAKMP_CFG_LOGOUT	2
 static int isakmp_cfg_accounting(struct ph1handle *, int);
+#ifdef HAVE_LIBRADIUS
+static int isakmp_cfg_accounting_radius(struct ph1handle *, int);
+#endif
 
 /* 
  * Handle an ISAKMP config mode packet
@@ -123,7 +130,7 @@ isakmp_cfg_r(iph1, msg)
 {
 	struct isakmp *packet;
 	struct isakmp_gen *ph;
-	size_t tlen;
+	int tlen;
 	char *npp;
 	int np;
 	vchar_t *dmsg;
@@ -296,7 +303,7 @@ isakmp_cfg_reply(iph1, attrpl)
 	struct isakmp_pl_attr *attrpl;
 {
 	struct isakmp_data *attr;
-	size_t tlen;
+	int tlen;
 	size_t alen;
 	char *npp;
 	int type;
@@ -433,7 +440,7 @@ isakmp_cfg_request(iph1, attrpl)
 	struct isakmp_pl_attr *attrpl;
 {
 	struct isakmp_data *attr;
-	size_t tlen;
+	int tlen;
 	size_t alen;
 	char *npp;
 	vchar_t *payload;
@@ -579,7 +586,7 @@ isakmp_cfg_set(iph1, attrpl)
 	struct isakmp_pl_attr *attrpl;
 {
 	struct isakmp_data *attr;
-	size_t tlen;
+	int tlen;
 	size_t alen;
 	char *npp;
 	vchar_t *payload;
@@ -964,8 +971,10 @@ isakmp_cfg_send(iph1, payload, np, flags, new_exchange)
 	iph2->src = dupsaddr(iph1->local);
 	switch (iph1->remote->sa_family) {
 	case AF_INET:
+#ifndef ENABLE_NATT
 		((struct sockaddr_in *)iph2->dst)->sin_port = 0;
 		((struct sockaddr_in *)iph2->src)->sin_port = 0;
+#endif
 		break;
 #ifdef INET6
 	case AF_INET6:
@@ -1146,20 +1155,14 @@ isakmp_cfg_getport(iph1)
 	unsigned int i;
 	size_t size = isakmp_cfg_config.pool_size;
 
-	/* For first time we use it */
 	if (isakmp_cfg_config.port_pool == NULL) {
-		isakmp_cfg_config.port_pool = racoon_malloc(size);
-		if (isakmp_cfg_config.port_pool == NULL) {
-			plog(LLV_ERROR, LOCATION, NULL, 
-			    "Cannot allocate memory\n");
-			return -1;
-		}
-
-		memset(isakmp_cfg_config.port_pool, 0, size);
+		plog(LLV_ERROR, LOCATION, NULL,
+		    "isakmp_cfg_config.port_pool == NULL\n");
+		return -1;
 	}
 
 	for (i = 0; i < size; i++) {
-		if (isakmp_cfg_config.port_pool[i] == 0)
+		if (isakmp_cfg_config.port_pool[i].used == 0)
 			break;
 	}
 
@@ -1169,7 +1172,7 @@ isakmp_cfg_getport(iph1)
 			return -1;
 	}
 
-	isakmp_cfg_config.port_pool[i] = 1;
+	isakmp_cfg_config.port_pool[i].used = 1;
 
 	plog(LLV_INFO, LOCATION, NULL, "Using port %d\n", i);
 
@@ -1190,14 +1193,19 @@ isakmp_cfg_putport(iph1, index)
 		return -1;
 	}
 
-	if (isakmp_cfg_config.port_pool[index] == 0) {
+	if (isakmp_cfg_config.port_pool[index].used == 0) {
 		plog(LLV_ERROR, LOCATION, NULL, 
 		    "Attempt to release an unallocated address (port %d)\n",
 		    index);
 		return -1;
 	}
 
-	isakmp_cfg_config.port_pool[index] = 0;
+#ifdef HAVE_LIBPAM
+	/* Cleanup PAM status associated with the port */
+	if (isakmp_cfg_config.authsource == ISAKMP_CFG_AUTH_PAM)
+		privsep_cleanup_pam(index);
+#endif
+	isakmp_cfg_config.port_pool[index].used = 0;
 	iph1->mode_cfg->flags &= ISAKMP_CFG_PORT_ALLOCATED;
 
 	plog(LLV_INFO, LOCATION, NULL, "Released port %d\n", index);
@@ -1205,48 +1213,121 @@ isakmp_cfg_putport(iph1, index)
 	return 0;
 }
 
-/* Accounting, only for Radius yet */
+#ifdef HAVE_LIBPAM
+void
+cleanup_pam(port)
+	int port;
+{
+	if (isakmp_cfg_config.port_pool[port].pam != NULL) {
+		pam_end(isakmp_cfg_config.port_pool[port].pam, PAM_SUCCESS);
+		isakmp_cfg_config.port_pool[port].pam = NULL;
+	}
+
+	return;
+}
+#endif
+
+/* Accounting, only for RADIUS or PAM */
 static int
 isakmp_cfg_accounting(iph1, inout)
 	struct ph1handle *iph1;
 	int inout;
 {
+#ifdef HAVE_LIBPAM
+	if (isakmp_cfg_config.accounting == ISAKMP_CFG_ACCT_PAM)
+		return privsep_accounting_pam(iph1->mode_cfg->port, 
+		    inout);
+#endif 
 #ifdef HAVE_LIBRADIUS
-	static struct rad_handle *radius_state = NULL;
+	if (isakmp_cfg_config.accounting == ISAKMP_CFG_ACCT_RADIUS)
+		return isakmp_cfg_accounting_radius(iph1, inout);
+#endif
+	return 0;
+}
 
-	if (isakmp_cfg_config.accounting != ISAKMP_CFG_ACCT_RADIUS)
-		return 0;
+#ifdef HAVE_LIBPAM
+int 
+isakmp_cfg_accounting_pam(port, inout)
+	int port;
+	int inout;
+{
+	int error = 0;
+	pam_handle_t *pam;
 
+	if (isakmp_cfg_config.port_pool == NULL) {
+		plog(LLV_ERROR, LOCATION, NULL, 
+		    "isakmp_cfg_config.port_pool == NULL\n");
+		return -1;
+	}
+	
+	pam = isakmp_cfg_config.port_pool[port].pam;
+	if (pam == NULL) {
+		plog(LLV_ERROR, LOCATION, NULL, "pam handle is NULL\n");
+		return -1;
+	}
+
+	switch (inout) {
+	case ISAKMP_CFG_LOGIN:
+		error = pam_open_session(pam, 0);
+		break;
+	case ISAKMP_CFG_LOGOUT:
+		error = pam_close_session(pam, 0);
+		pam_end(pam, error);
+		isakmp_cfg_config.port_pool[port].pam = NULL;
+		break;
+	default:
+		plog(LLV_ERROR, LOCATION, NULL, "Unepected inout\n");
+		break;
+	}
+	
+	if (error != 0) {
+		plog(LLV_ERROR, LOCATION, NULL,
+		    "pam_open_session/pam_close_session failed: %s\n",
+		    pam_strerror(pam, error)); 
+		return -1;
+        }
+
+	return 0;
+}
+#endif /* HAVE_LIBPAM */
+
+#ifdef HAVE_LIBRADIUS
+static int
+isakmp_cfg_accounting_radius(iph1, inout)
+	struct ph1handle *iph1;
+	int inout;
+{
 	/* For first time use, initialize Radius */
-	if (radius_state == NULL) {
-		if ((radius_state = rad_acct_open()) == NULL) {
+	if (radius_acct_state == NULL) {
+		if ((radius_acct_state = rad_acct_open()) == NULL) {
 			plog(LLV_ERROR, LOCATION, NULL,
 			    "Cannot init librradius\n");
 			return -1;
 		}
 
-		if (rad_config(radius_state, NULL) != 0) {
+		if (rad_config(radius_acct_state, NULL) != 0) {
 			 plog(LLV_ERROR, LOCATION, NULL,
 			     "Cannot open librarius config file: %s\n",
-			     rad_strerror(radius_state));
-			  rad_close(radius_state);
-			  radius_state = NULL;
+			     rad_strerror(radius_acct_state));
+			  rad_close(radius_acct_state);
+			  radius_acct_state = NULL;
 			  return -1;
 		}
 	}
 
-	if (rad_create_request(radius_state, RAD_ACCOUNTING_REQUEST) != 0) {
+	if (rad_create_request(radius_acct_state, 
+	    RAD_ACCOUNTING_REQUEST) != 0) {
 		plog(LLV_ERROR, LOCATION, NULL,
 		    "rad_create_request failed: %s\n",
-		    rad_strerror(radius_state));
+		    rad_strerror(radius_acct_state));
 		return -1;
 	}
 
-	if (rad_put_string(radius_state, RAD_USER_NAME, 
+	if (rad_put_string(radius_acct_state, RAD_USER_NAME, 
 	    iph1->mode_cfg->login) != 0) {
 		plog(LLV_ERROR, LOCATION, NULL,
 		    "rad_put_string failed: %s\n",
-		    rad_strerror(radius_state));
+		    rad_strerror(radius_acct_state));
 		return -1;
 	}
 
@@ -1262,41 +1343,43 @@ isakmp_cfg_accounting(iph1, inout)
 		break;
 	}
 
-	if (rad_put_addr(radius_state, 
+	if (rad_put_addr(radius_acct_state, 
 	    RAD_FRAMED_IP_ADDRESS, iph1->mode_cfg->addr4) != 0) {
 		plog(LLV_ERROR, LOCATION, NULL,
 		    "rad_put_addr failed: %s\n",
-		    rad_strerror(radius_state));
+		    rad_strerror(radius_acct_state));
 		return -1;
 	}
 
-	if (rad_put_addr(radius_state, 
+	if (rad_put_addr(radius_acct_state, 
 	    RAD_LOGIN_IP_HOST, iph1->mode_cfg->addr4) != 0) {
 		plog(LLV_ERROR, LOCATION, NULL,
 		    "rad_put_addr failed: %s\n",
-		    rad_strerror(radius_state));
+		    rad_strerror(radius_acct_state));
 		return -1;
 	}
 
-	if (rad_put_int(radius_state, RAD_ACCT_STATUS_TYPE, inout) != 0) {
+	if (rad_put_int(radius_acct_state, RAD_ACCT_STATUS_TYPE, inout) != 0) {
 		plog(LLV_ERROR, LOCATION, NULL,
 		    "rad_put_int failed: %s\n",
-		    rad_strerror(radius_state));
+		    rad_strerror(radius_acct_state));
 		return -1;
 	}
 
-	if (isakmp_cfg_radius_common(radius_state, iph1->mode_cfg->port) != 0)
+	if (isakmp_cfg_radius_common(radius_acct_state, 
+	    iph1->mode_cfg->port) != 0)
 		return -1;
 
-	if (rad_send_request(radius_state) != RAD_ACCOUNTING_RESPONSE) {
+	if (rad_send_request(radius_acct_state) != RAD_ACCOUNTING_RESPONSE) {
 		plog(LLV_ERROR, LOCATION, NULL,
 		    "rad_send_request failed: %s\n",
-		    rad_strerror(radius_state));
+		    rad_strerror(radius_acct_state));
 		return -1;
 	}
-#endif
+
 	return 0;
 }
+#endif /* HAVE_LIBRADIUS */
 
 /*
  * Attributes common to all RADIUS requests
@@ -1429,9 +1512,11 @@ isakmp_cfg_getaddr4(attr, ip)
 	return;
 }
 
-void 
-isakmp_cfg_setenv(iph1)
+int
+isakmp_cfg_setenv(iph1, envp, envc)
 	struct ph1handle *iph1; 
+	char ***envp;
+	int *envc;
 {
 #define IP_MAX 40
 	char addrstr[IP_MAX];
@@ -1449,7 +1534,11 @@ isakmp_cfg_setenv(iph1)
 		    addrstr, IP_MAX);
 	} else
 		addrstr[0] = '\0';
-	setenv("INTERNAL_ADDR4", addrstr, 1);
+
+	if (script_env_append(envp, envc, "INTERNAL_ADDR4", addrstr) != 0) {
+		plog(LLV_ERROR, LOCATION, NULL, "Cannot set INTERNAL_ADDR4\n");
+		return -1;
+	}
 
 	/* Internal IPv4 mask */
 	if (iph1->mode_cfg->flags & ISAKMP_CFG_GOT_MASK4) 
@@ -1457,7 +1546,11 @@ isakmp_cfg_setenv(iph1)
 		    addrstr, IP_MAX);
 	else
 		addrstr[0] = '\0';
-	setenv("INTERNAL_MASK4", addrstr, 1);
+
+	if (script_env_append(envp, envc, "INTERNAL_MASK4", addrstr) != 0) { 
+		plog(LLV_ERROR, LOCATION, NULL, "Cannot set INTERNAL_MASK4\n");
+		return -1;
+	}
 
 	/* Internal IPv4 DNS */
 	if (iph1->mode_cfg->flags & ISAKMP_CFG_GOT_DNS4) 
@@ -1465,7 +1558,11 @@ isakmp_cfg_setenv(iph1)
 		    addrstr, IP_MAX);
 	else
 		addrstr[0] = '\0';
-	setenv("INTERNAL_DNS4", addrstr, 1);
+
+	if (script_env_append(envp, envc, "INTERNAL_DNS4", addrstr) != 0) { 
+		plog(LLV_ERROR, LOCATION, NULL, "Cannot set INTERNAL_DNS4\n");
+		return -1;
+	}
 
 	/* Internal IPv4 WINS */
 	if (iph1->mode_cfg->flags & ISAKMP_CFG_GOT_WINS4) 
@@ -1473,7 +1570,11 @@ isakmp_cfg_setenv(iph1)
 		    addrstr, IP_MAX);
 	else
 		addrstr[0] = '\0';
-	setenv("INTERNAL_WINS4", addrstr, 1);
 
-	return;
+	if (script_env_append(envp, envc, "INTERNAL_WINS4", addrstr) != 0) { 
+		plog(LLV_ERROR, LOCATION, NULL, "Cannot set INTERNAL_WINS4\n");
+		return -1;
+	}
+
+	return 0;
 }
