@@ -1,4 +1,4 @@
-/* $Id: localconf.h,v 1.7 2004/11/18 14:27:06 ludvigm Exp $ */
+/* $Id: localconf.h,v 1.9.2.2 2005/03/16 23:18:43 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -40,7 +40,8 @@
 #define LC_PATHTYPE_PSK		1
 #define LC_PATHTYPE_CERT	2
 #define LC_PATHTYPE_BACKUPSA	3
-#define LC_PATHTYPE_MAX		4
+#define LC_PATHTYPE_SCRIPT	4
+#define LC_PATHTYPE_MAX		5
 
 #define LC_DEFAULT_PAD_MAXSIZE		20
 #define LC_DEFAULT_PAD_RANDOM		TRUE
@@ -58,9 +59,16 @@
 
 #define LC_IDENTTYPE_MAX	5	/* XXX */
 
+#define	LC_GSSENC_UTF16LE	0	/* GSS ID in UTF-16LE */
+#define	LC_GSSENC_LATIN1	1	/* GSS ID in ISO-Latin-1 */
+#define	LC_GSSENC_MAX		2
+
 struct localconf {
 	char *racoon_conf;		/* configuration filename */
 
+	uid_t uid;
+	gid_t gid;
+	char *chroot;			/* chroot path */
 	u_int16_t port_isakmp;		/* port for isakmp as default */
 	u_int16_t port_isakmp_natt;	/* port for NAT-T use */
 	u_int16_t port_admin;		/* port for admin */
@@ -105,6 +113,8 @@ struct localconf {
 		 * usually uses this format.  If the option, 'complex_bundle'
 		 * is enable, racoon uses old format.
 		 */
+
+	int gss_id_enc;			/* GSS ID encoding to use */
 };
 
 extern struct localconf *lcconf;
@@ -116,5 +126,6 @@ extern vchar_t *getpskbyaddr __P((struct sockaddr *));
 extern void getpathname __P((char *, int, int, const char *));
 extern int sittype2doi __P((int));
 extern int doitype2doi __P((int));
+extern vchar_t *getpsk __P((const char *, const int)); 
 
 #endif /* _LOCALCONF_H */
