@@ -1,4 +1,4 @@
-/* $Id: main.c,v 1.12 2004/09/21 14:35:24 ludvigm Exp $ */
+/* $Id: main.c,v 1.14.2.2 2005/02/23 12:18:40 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -64,6 +64,10 @@
 
 #include "cfparse_proto.h"
 #include "isakmp_var.h"
+#ifdef HAVE_LIBRADIUS
+#include "isakmp.h"
+#include "isakmp_xauth.h"
+#endif
 #include "remoteconf.h"
 #include "localconf.h"
 #include "session.h"
@@ -73,7 +77,7 @@
 #include "backupsa.h"
 #include "vendorid.h"
 
-#include "../../package_version.h"
+#include "package_version.h"
 
 int f_local = 0;	/* local test mode.  behave like a wall. */
 int vflag = 1;		/* for print-isakmp.c */
@@ -192,6 +196,13 @@ main(ac, av)
 		errx(1, "failed to parse configuration file.");
 	restore_params();
 
+#ifdef HAVE_LIBRADIUS
+	if (xauth_radius_init() != 0) {
+		errx(1, "could not initialize libradius");
+		/* NOTREACHED*/
+	}
+#endif
+
 	if (dump_config)
 		dumprmconf ();
 
