@@ -1,4 +1,4 @@
-/* $Id: oakley.c,v 1.15.2.2 2005/03/01 09:51:27 vanhu Exp $ */
+/* $Id: oakley.c,v 1.17.2.1 2005/03/01 09:51:48 vanhu Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -66,6 +66,8 @@
 #include "isakmp_cfg.h" 
 #endif                
 #include "oakley.h"
+#include "admin.h"
+#include "privsep.h"
 #include "localconf.h"
 #include "remoteconf.h"
 #include "policy.h"
@@ -1714,7 +1716,7 @@ oakley_getsign(iph1)
 		getpathname(path, sizeof(path),
 			LC_PATHTYPE_CERT,
 			iph1->rmconf->myprivfile);
-		privkey = eay_get_pkcs1privkey(path);
+		privkey = privsep_eay_get_pkcs1privkey(path);
 		if (privkey == NULL) {
 			plog(LLV_ERROR, LOCATION, NULL,
 				"failed to get private key.\n");
