@@ -1,4 +1,4 @@
-/* $Id: pfkey.c,v 1.27.2.5 2005/04/21 08:57:17 monas Exp $ */
+/* $Id: pfkey.c,v 1.31.2.5 2005/05/03 20:47:24 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -92,6 +92,7 @@
 #include "sainfo.h"
 #include "proposal.h"
 #include "admin.h"
+#include "privsep.h"
 #include "strnames.h"
 #include "backupsa.h"
 #include "gcmalloc.h"
@@ -285,7 +286,7 @@ pfkey_dump_sadb(satype)
 	size_t bl, ml;
 	int len;
 
-	if ((s = pfkey_open()) < 0) {
+	if ((s = privsep_pfkey_open()) < 0) {
 		plog(LLV_ERROR, LOCATION, NULL,
 			"libipsec failed pfkey open: %s\n",
 			ipsec_strerror());
@@ -336,7 +337,7 @@ done:
 	if (msg)
 		racoon_free(msg);
 	if (s >= 0)
-		close(s);
+		privsep_pfkey_close(s);
 	return buf;
 }
 
@@ -387,7 +388,7 @@ pfkey_init()
 {
 	int i, reg_fail;
 
-	if ((lcconf->sock_pfkey = pfkey_open()) < 0) {
+	if ((lcconf->sock_pfkey = privsep_pfkey_open()) < 0) {
 		plog(LLV_ERROR, LOCATION, NULL,
 			"libipsec failed pfkey open (%s)\n", ipsec_strerror());
 		return -1;
@@ -1055,6 +1056,7 @@ pk_sendupdate(iph2)
 			natt.sport = extract_port (iph2->ph1->remote);
 			natt.dport = extract_port (iph2->ph1->local);
 			natt.oa = NULL;		// FIXME: Here comes OA!!!
+			natt.frag = iph2->ph1->rmconf->esp_frag;
 		}
 		else
 			memset (&natt, 0, sizeof (natt));
@@ -1072,7 +1074,8 @@ pk_sendupdate(iph2)
 				e_type, e_keylen, a_type, a_keylen, flags,
 				0, lifebyte, iph2->approval->lifetime, 0,
 				iph2->seq,
-				natt.type, natt.sport, natt.dport, natt.oa) < 0) {
+				natt.type, natt.sport, natt.dport, natt.oa,
+				natt.frag) < 0) {
 			plog(LLV_ERROR, LOCATION, NULL,
 				"libipsec failed send update_nat (%s)\n",
 				ipsec_strerror());
@@ -1346,6 +1349,7 @@ pk_sendadd(iph2)
 			natt.sport = extract_port (iph2->ph1->local);
 			natt.dport = extract_port (iph2->ph1->remote);
 			natt.oa = NULL;		// FIXME: Here comes OA!!!
+			natt.frag = iph2->ph1->rmconf->esp_frag;
 		}
 		else
 			memset (&natt, 0, sizeof (natt));
@@ -1363,7 +1367,8 @@ pk_sendadd(iph2)
 				e_type, e_keylen, a_type, a_keylen, flags,
 				0, lifebyte, iph2->approval->lifetime, 0,
 				iph2->seq,
-				natt.type, natt.sport, natt.dport, natt.oa) < 0) {
+				natt.type, natt.sport, natt.dport, natt.oa,
+				natt.frag) < 0) {
 			plog(LLV_ERROR, LOCATION, NULL,
 				"libipsec failed send add_nat (%s)\n",
 				ipsec_strerror());
@@ -1606,6 +1611,7 @@ pk_recvacquire(mhp)
 	struct secpolicy *sp_out = NULL, *sp_in = NULL;
 #define MAXNESTEDSA	5	/* XXX */
 	struct ph2handle *iph2[MAXNESTEDSA];
+	struct sockaddr *src, *dst;
 	int n;	/* # of phase 2 handler */
 
 	/* ignore this message because of local test mode. */
@@ -1623,6 +1629,8 @@ pk_recvacquire(mhp)
 	}
 	msg = (struct sadb_msg *)mhp[0];
 	xpl = (struct sadb_x_policy *)mhp[SADB_X_EXT_POLICY];
+	src = PFKEY_ADDR_SADDR(mhp[SADB_EXT_ADDRESS_SRC]);
+	dst = PFKEY_ADDR_SADDR(mhp[SADB_EXT_ADDRESS_DST]);
 
 	/* ignore if type is not IPSEC_POLICY_IPSEC */
 	if (xpl->sadb_x_policy_type != IPSEC_POLICY_IPSEC) {
@@ -1687,7 +1695,7 @@ pk_recvacquire(mhp)
 	 *       has to prcesss such a acquire message because racoon may
 	 *       lost the expire message.
 	 */
-	iph2[0] = getph2byspid(xpl->sadb_x_policy_id);
+	iph2[0] = getph2byid(src, dst, xpl->sadb_x_policy_id);
 	if (iph2[0] != NULL) {
 		if (iph2[0]->status < PHASE2ST_ESTABLISHED) {
 			plog(LLV_DEBUG, LOCATION, NULL,
@@ -2833,13 +2841,13 @@ sadbsecas2str(src, dst, proto, spi, mode)
 	p += i;
 	blen -= i;
 
-	i = snprintf(p, blen, "%s->", saddrwop2str(src));
+	i = snprintf(p, blen, "%s->", saddr2str(src));
 	if (i < 0 || i >= blen)
 		return NULL;
 	p += i;
 	blen -= i;
 
-	i = snprintf(p, blen, "%s ", saddrwop2str(dst));
+	i = snprintf(p, blen, "%s ", saddr2str(dst));
 	if (i < 0 || i >= blen)
 		return NULL;
 	p += i;
