@@ -1,4 +1,4 @@
-/* $Id: proposal.c,v 1.13 2004/09/13 14:09:19 ludvigm Exp $ */
+/* $Id: proposal.c,v 1.13.8.3 2005/05/20 01:16:00 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -220,6 +220,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 		newpp->lifebyte = pp1->lifebyte;
 		newpp->pfs_group = pp1->pfs_group;
 		break;
+
 	case PROP_CHECK_STRICT:
 		if (pp1->lifetime > pp2->lifetime) {
 			plog(LLV_ERROR, LOCATION, NULL,
@@ -248,6 +249,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 		}
 		newpp->pfs_group = pp1->pfs_group;
 		break;
+
 	case PROP_CHECK_CLAIM:
 		/* lifetime */
 		if (pp1->lifetime <= pp2->lifetime) {
@@ -274,6 +276,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 
     		goto prop_pfs_check;
 		break;
+
 	case PROP_CHECK_EXACT:
 		if (pp1->lifetime != pp2->lifetime) {
 			plog(LLV_ERROR, LOCATION, NULL,
@@ -282,6 +285,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 				(int)pp2->lifetime, (int)pp1->lifetime);
 			goto err;
 		}
+
 		if (pp1->lifebyte != pp2->lifebyte) {
 			plog(LLV_ERROR, LOCATION, NULL,
 				"lifebyte mismatched: "
@@ -300,6 +304,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 		newpp->lifebyte = pp1->lifebyte;
 		newpp->pfs_group = pp1->pfs_group;
 		break;
+
 	default:
 		plog(LLV_ERROR, LOCATION, NULL,
 			"invalid pcheck_level why?.\n");
@@ -365,8 +370,8 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 			if (pr1->spisize == sizeof(u_int16_t) &&
 			    pr2->spisize == sizeof(u_int32_t)) {
 				spisizematch = 1;
-			} else if (pr1->spisize == sizeof(u_int16_t) &&
-				 pr2->spisize == sizeof(u_int32_t)) {
+			} else if (pr2->spisize == sizeof(u_int16_t) &&
+				 pr1->spisize == sizeof(u_int32_t)) {
 				spisizematch = 1;
 			}
 			if (spisizematch) {
@@ -1167,3 +1172,15 @@ end:
 	free_proppair(pair);
 	return error;
 }
+
+int
+tunnel_mode_prop(p)
+	struct saprop *p;
+{
+	struct saproto *pr;
+
+	for (pr = p->head; pr; pr = pr->next)
+		if (pr->encmode == IPSECDOI_ATTR_ENC_MODE_TUNNEL)
+			return 1;
+	return 0;
+}
