@@ -1,4 +1,4 @@
-/* $Id: remoteconf.c,v 1.24 2004/12/01 10:59:41 vanhu Exp $ */
+/* $Id: remoteconf.c,v 1.26.2.4 2005/05/20 00:37:41 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -37,6 +37,8 @@
 #include <sys/queue.h>
 
 #include <netinet/in.h>
+#include <netinet/in_systm.h>
+#include <netinet/ip.h>
 
 #ifndef HAVE_NETINET6_IPSEC
 #include <netinet/ipsec.h>
@@ -75,9 +77,10 @@
 static TAILQ_HEAD(_rmtree, remoteconf) rmtree;
 
 /* 
- * Script hook names
+ * Script hook names and script hook paths
  */
 char *script_names[SCRIPT_MAX + 1] = { "phase1_up", "phase1_down" };
+vchar_t *script_paths = NULL;
 
 /*%%%*/
 /*
@@ -101,6 +104,16 @@ getrmconf_strict(remote, allow_anon)
 
 	withport = 0;
 
+#ifndef ENABLE_NATT
+	/* 
+	 * We never have ports set in our remote configurations, but when
+	 * NAT-T is enabled, the kernel can have policies with ports and
+	 * send us an acquire message for a destination that has a port set.
+	 * If we do this port check here, we don't find the remote config.
+	 *
+	 * In an ideal world, we would be able to have remote conf with
+	 * port, and the port could be a wildcard. That test could be used.
+	 */
 	switch (remote->sa_family) {
 	case AF_INET:
 		if (((struct sockaddr_in *)remote)->sin_port != IPSEC_PORT_ANY)
@@ -120,6 +133,7 @@ getrmconf_strict(remote, allow_anon)
 			"invalid family: %d\n", remote->sa_family);
 		exit(1);
 	}
+#endif /* ENABLE_NATT */
 
 	if (remote->sa_family == AF_UNSPEC)
 		snprintf (buf, sizeof(buf), "%s", "anonymous");
@@ -169,6 +183,7 @@ struct remoteconf *
 newrmconf()
 {
 	struct remoteconf *new;
+	int i;
 
 	new = racoon_calloc(1, sizeof(*new));
 	if (new == NULL)
@@ -184,6 +199,7 @@ newrmconf()
 	new->nonce_size = DEFAULT_NONCE_SIZE;
 	new->passive = FALSE;
 	new->ike_frag = FALSE;
+	new->esp_frag = IP_MAXPACKET;
 	new->ini_contact = TRUE;
 	new->mode_cfg = FALSE;
 	new->pcheck_level = PROP_CHECK_STRICT;
@@ -196,7 +212,8 @@ newrmconf()
 	new->send_cert = TRUE;
 	new->send_cr = TRUE;
 	new->support_proxy = FALSE;
-	bzero(&new->script[0], sizeof(char *) * (SCRIPT_MAX + 1));
+	for (i = 0; i <= SCRIPT_MAX; i++)
+		new->script[i] = -1;
 	new->gen_policy = FALSE;
 	new->retry_counter = lcconf->retry_counter;
 	new->retry_interval = lcconf->retry_interval;
@@ -540,6 +557,7 @@ dump_rmconf_single (struct remoteconf *p, void *data)
 		s_switch (p->passive));
 	plog(LLV_INFO, LOCATION, NULL, "\tike_frag %s;\n",
 		s_switch (p->ike_frag));
+	plog(LLV_INFO, LOCATION, NULL, "\tesp_frag %d;\n", p->esp_frag);
 	plog(LLV_INFO, LOCATION, NULL, "\tinitial_contact %s;\n",
 		s_switch (p->ini_contact));
 	plog(LLV_INFO, LOCATION, NULL, "\tgenerate_policy %s;\n",
@@ -594,3 +612,85 @@ newidspec()
 	return new;
 }
 
+int
+script_path_add(path)
+	vchar_t *path;
+{
+	char *script_dir;
+	vchar_t *new_storage;
+	vchar_t *new_path;
+	vchar_t **sp;
+	size_t len;
+	size_t size;
+
+	script_dir = lcconf->pathinfo[LC_PATHTYPE_SCRIPT];
+
+	/* Try to find the script in the script directory */
+	if ((path->v[0] != '/') && (script_dir != NULL)) {
+		len = strlen(script_dir) + sizeof("/") + path->l + 1;
+
+		if ((new_path = vmalloc(len)) == NULL) {
+			plog(LLV_ERROR, LOCATION, NULL,
+			    "Cannot allocate memory: %s\n", strerror(errno));
+			return -1;
+		}
+
+		new_path->v[0] = '\0';
+		(void)strncat(new_path->v, script_dir, len);
+		(void)strncat(new_path->v, "/", len);
+		(void)strncat(new_path->v, path->v, len);
+
+		vfree(path);
+		path = new_path;
+	}
+
+	/* First time, initialize */
+	if (script_paths == NULL)
+		len = sizeof(vchar_t *);
+	else
+		len = script_paths->l;
+
+	/* Add a slot for a new path */
+	len += sizeof(vchar_t *);
+	if ((new_storage = vrealloc(script_paths, len)) == NULL) {
+		plog(LLV_ERROR, LOCATION, NULL,
+		    "Cannot allocate memory: %s\n", strerror(errno));
+		return -1;
+	}
+	script_paths = new_storage;
+
+	size = len / sizeof(vchar_t *);
+	sp = (vchar_t **)script_paths->v;	
+	sp[size - 1] = NULL;
+	sp[size - 2] = path;
+
+	return (size - 2);
+}
+
+struct isakmpsa *
+dupisakmpsa(sa)
+	struct isakmpsa *sa;
+{
+	struct isakmpsa *res = NULL;
+
+	if (sa == NULL)
+		return NULL;
+
+	res = newisakmpsa();
+	if(res == NULL)
+		return NULL;
+
+	*res = *sa;
+#ifdef HAVE_GSSAPI
+	/* 
+	 * XXX gssid
+	 */
+#endif
+	res->next=NULL;
+
+	if (sa->dhgrp != NULL)
+		oakley_setdhgroup(sa->dh_group, &(res->dhgrp));
+
+	return res;
+
+}
