@@ -67,7 +67,7 @@
 
 #include "config.h"
 #include "libpfkey.h"
-#include "../../package_version.h"
+#include "package_version.h"
 
 #define strlcpy(d,s,l) (strncpy(d,s,l), (d)[(l)-1] = '\0')
 
@@ -106,6 +106,7 @@ int f_policy = 0;
 int f_hexdump = 0;
 int f_tflag = 0;
 int f_notreally = 0;
+int f_withports = 0;
 #ifdef HAVE_POLICY_FWD
 int f_rfcmode = 1;
 #define RK_OPTS "rk"
@@ -173,7 +174,7 @@ main(argc, argv)
 
 	thiszone = gmt2local(0);
 
-	while ((c = getopt(argc, argv, "acdf:HlnvxDFPhVrk")) != -1) {
+	while ((c = getopt(argc, argv, "acdf:HlnvxDFPphVrk?")) != -1) {
 		switch (c) {
 		case 'c':
 			f_mode = MODE_STDIN;
@@ -200,6 +201,9 @@ main(argc, argv)
 		case 'n':
 			f_notreally = 1;
 			break;
+#ifdef __NetBSD__
+		case 'h':
+#endif
 		case 'H':
 			f_hexdump = 1;
 			break;
@@ -210,6 +214,9 @@ main(argc, argv)
 		case 'P':
 			f_policy = 1;
 			break;
+		case 'p':
+			f_withports = 1;
+			break;
 		case 'v':
 			f_verbose = 1;
 			break;
@@ -231,7 +238,10 @@ main(argc, argv)
 			usage(1);
 			break;
 			/*NOTREACHED*/
+#ifndef __NetBSD__
 		case 'h':
+#endif
+		case '?':
 		default:
 			usage(0);
 			/*NOTREACHED*/
@@ -600,11 +610,17 @@ postproc(msg, len)
 		break;
 
 	case SADB_X_SPDGET:
-		pfkey_spdump(msg);
+		if (f_withports) 
+			pfkey_spdump_withports(msg);
+		else
+			pfkey_spdump(msg);
 		break;
 
 	case SADB_X_SPDDUMP:
-		pfkey_spdump(msg);
+		if (f_withports) 
+			pfkey_spdump_withports(msg);
+		else
+			pfkey_spdump(msg);
 		if (msg->sadb_msg_seq == 0) break;
 		msg = (struct sadb_msg *)((caddr_t)msg +
 				     PFKEY_UNUNIT64(msg->sadb_msg_len));
